#include "Main.h"
#include "PakFile_v3.h"

using namespace std;

Statistics mpakDump_v3(FILE* f, const std::string& name)
{
    u32 i;
    MPakHeader h;
    Statistics out;
    fseek(f, 0x00, SEEK_SET);

    fread(&h, sizeof(h), 1, f);
    toWORD(h.version1);
    toWORD(h.version2);
    toDWORD(h.size);

    assert(h.version1 == 0);
    assert(h.version2 == 2);

    fseek(f, 0x40, SEEK_SET);

    //read section entry header
    EntryHeader seh;
    fread(&seh, sizeof(seh), 1, f);
    toDWORD(seh.entryCount);

    cout << hex;

    vector<SectionEntry> se;
    se.resize(seh.entryCount);

    //output section entries
    if (bVerbose)
    {
        cout << string(70, '-') << endl
        << seh.entryCount << " section entries: " << endl;
    }
    for (i = 0; i < seh.entryCount; ++i)
    {
        fread(&se[i], sizeof(SectionEntry), 1, f);
        toDWORD(se[i].dataLength);
        if (bVerbose)
            cout << string(se[i].tag, 4) << " " << se[i].dataLength << endl;
    }

    fseek(f, 0x80, SEEK_SET);

    //read name entry header
    EntryHeader neh;
    fread(&neh, sizeof(neh), 1, f);
    toDWORD(neh.entryCount);

    vector<NameEntry> ne;
    ne.resize(neh.entryCount);
    vector<string> names;
    names.resize(neh.entryCount);
    cout << hex;

    //output name entries
    if (bVerbose)
    {
        cout << string(70, '-') << endl
        << neh.entryCount << " name entries: " << endl;
    }
    for (i = 0; i < neh.entryCount; ++i)
    {
        char name[1000] = {0};
        fscanf(f, "%[^\0]", name);
        fseek(f, 0x01, SEEK_CUR);
        //NameEntry ne;
        fread(&ne[i], sizeof(NameEntry), 1, f);
        toDWORD(ne[i].id1);
        toWORD(ne[i].id2);
        names[i] = name;

        if (bVerbose)
            cout << string(ne[i].tag, 4) << " " << ne[i].id1 << ne[i].id2 << " " << string(name) << endl;
    }
    fseek(f, 0x80 + se[0].dataLength, SEEK_SET);

    const u32 dataStart = 0x80 + se[0].dataLength + se[1].dataLength;

    //read entry header
    EntryHeader eh;
    fread(&eh, sizeof(eh), 1, f);
    toDWORD(eh.entryCount);

    //output entries
    if (bVerbose)
    {
        cout << string(70, '-') << endl
        << dec << eh.entryCount << " entries: " << endl;
    }
    int count = 0;
    for (i = 0; i < eh.entryCount; ++i)
    {
        Entry e;
        string fname;
        fread(&e, sizeof(e), 1, f);
        toDWORD(e.comp);
        toDWORD(e.id1);
        toWORD(e.id2);
        toWORD(e.id3);
        toDWORD(e.length);
        toDWORD(e.offset);

        for (u32 ii = 0; ii < neh.entryCount; ii++)
        {
            if (e.id1 == ne[ii].id1 && e.id2 == ne[ii].id2)
            {
                fname = names[ii];
                break;
            }
        }

        assert(e.comp == 0 || e.comp == 1);
        count += e.comp;

        //dump file
        std::ostringstream nameStream;
        nameStream << name << "\\" << e.comp << "_" << hex << e.id1 << e.id2 << e.id3 << "."
            << string(e.tag, 4);

        if (!bQuiet)
        {
            if (bVerbose)
                cout << hex << "Current File " << e.id1 << e.id2 << e.id3 << "." << string(e.tag, 4) << endl;
            else
                cout << "\rFile " << dec <<  fixed << (i+1) << " of " << eh.entryCount << " " << setprecision(2)<< ((float)((i+1)*100)/eh.entryCount) << "% Complete";
        }
        dumpFile(nameStream.str(), f, dataStart + e.offset, e.length);
    }
    cout << dec << endl;

    out.numFiles = eh.entryCount;
    out.numNames = neh.entryCount;
    out.numCompressed = count;
    return out;
}
